@@ -9,6 +9,6 @@ set -e
 $ipset n resize-test hash:ip hashsize 64
 for x in `seq 1 32`; do
    for y in `seq 1 255`; do
-      $ipset a resize-test 192.168.$x.$y
-   done
+      echo "a resize-test 192.168.$x.$y"
+   done | $ipset restore
 done
