@@ -45,12 +45,22 @@ add_tests() {
 	     `$cmd -t filter | grep ACCEPT | wc -l` -eq 3 ]; then
 	     	if [ -z "`which sendip`" ]; then
 	     		echo "sendip utility is missig: skipping $1 match and target tests"
-	     	elif [ -n "`netstat --protocol $1 -n | grep $2`" ]; then
-	     		echo "Our test network $2 in use: skipping $1 match and target tests"
+	     		return
+	     	elif [ -n "`which ss`" ]; then
+	     		if [ -n "`ss -f $1 -t -u -a | grep $2`" ]; then
+	     			echo "Our test network $2 in use: skipping $1 match and target tests"
+	     			return
+	     		fi
+	     	elif [ -n "`which netstat`" ]; then
+	     		if [ -n "`netstat --protocol $1 -n | grep $2`" ]; then
+	     			echo "Our test network $2 in use: skipping $1 match and target tests"
+	     			return
+	     		fi
 	     	else
-	     		tests="$tests $add"
+	     		echo "Cannot check test network, skipping $1 match and target tests"
+	     		return
 	     	fi
-	     	:
+	     	tests="$tests $add"
 	else
 		echo "You have got iptables rules: skipping $1 match and target tests"
 	fi
@@ -66,6 +76,7 @@ fi
 # Make sure the scripts are executable
 chmod a+x check_* *.sh
 
+failcount=0
 for types in $tests; do
     $ipset -X test >/dev/null 2>&1
     if [ -f $types ]; then
@@ -106,7 +117,8 @@ for types in $tests; do
 		echo "FAILED"
 		echo "Failed test: $cmd"
 		cat .foo.err
-		exit 1
+		let "failcount++"
+		break
 	fi
 	# sleep 1
     done < $filename
@@ -126,5 +138,9 @@ for x in $tests; do
 done
 rmmod ip_set >/dev/null 2>&1
 rm -f .foo*
-echo "All tests are passed"
-
+if [ "$failcount" -eq 0 ]; then
+	echo "All tests are passed"
+else
+	echo "$failcount tests have failed"
+	exit 1
+fi
